@@ -1,85 +1,39 @@
 //
-// "$Id: Fl_Bitmap.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id$"
 //
 // Bitmap drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2016 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
 //     http://www.fltk.org/str.php
 //
 
+/** \fn Fl_Bitmap::Fl_Bitmap(const char *array, int W, int H)
+  The constructors create a new bitmap from the specified bitmap data.*/
+
+/** \fn Fl_Bitmap::Fl_Bitmap(const unsigned char *array, int W, int H)
+  The constructors create a new bitmap from the specified bitmap data.*/
+
 #include <FL/Fl.H>
 #include <FL/x.H>
 #include <FL/fl_draw.H>
 #include <FL/Fl_Widget.H>
 #include <FL/Fl_Menu_Item.H>
 #include <FL/Fl_Bitmap.H>
+#include <FL/Fl_Printer.H>
 #include "flstring.h"
 
-#ifdef __APPLE_QD__ // MacOS bitmask functions
-Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *array) {
-  Rect srcRect;
-  srcRect.left = 0; srcRect.right = w;
-  srcRect.top = 0; srcRect.bottom = h;
-  GrafPtr savePort;
+#if defined(__APPLE_QUARTZ__)
 
-  GetPort(&savePort); // remember the current port
 
-  Fl_Bitmask gw;
-  NewGWorld( &gw, 1, &srcRect, 0L, 0L, 0 );
-  PixMapHandle pm = GetGWorldPixMap( gw );
-  if ( pm ) 
-  {
-    LockPixels( pm );
-    if ( *pm ) 
-    {
-      uchar *base = (uchar*)GetPixBaseAddr( pm );
-      if ( base ) 
-      {
-        PixMapPtr pmp = *pm;
-        // verify the parameters for direct memory write
-        if ( pmp->pixelType == 0 || pmp->pixelSize == 1 || pmp->cmpCount == 1 || pmp->cmpSize == 1 ) 
-        {
-          static uchar reverse[16] =	/* Bit reversal lookup table */
-          { 0x00, 0x88, 0x44, 0xcc, 0x22, 0xaa, 0x66, 0xee, 0x11, 0x99, 0x55, 0xdd, 0x33, 0xbb, 0x77, 0xff };
-          uchar *dst = base;
-          const uchar *src = array;
-          int rowBytesSrc = (w+7)>>3 ;
-          int rowPatch = (pmp->rowBytes&0x3fff) - rowBytesSrc;
-          for ( int j=0; j<h; j++,dst+=rowPatch )
-            for ( int i=0; i<rowBytesSrc; i++,src++ )
-              *dst++ = (reverse[*src & 0x0f] & 0xf0) | (reverse[(*src >> 4) & 0x0f] & 0x0f);
-        }
-      }
-      UnlockPixels( pm );
-    }
-  }
-
-  SetPort(savePort);
-  return gw;               /* tell caller we succeeded! */
-}
-
-void fl_delete_bitmask(Fl_Bitmask id) {
-  if (id) DisposeGWorld(id);
-}
-#elif defined(__APPLE_QUARTZ__)
 Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *array) {
   static uchar reverse[16] =    /* Bit reversal lookup table */
     { 0x00, 0x88, 0x44, 0xcc, 0x22, 0xaa, 0x66, 0xee, 
@@ -91,14 +45,18 @@ Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *array) {
     *dst++ = ((reverse[*src & 0x0f] & 0xf0) | (reverse[(*src >> 4) & 0x0f] & 0x0f))^0xff;
   }
   CGDataProviderRef srcp = CGDataProviderCreateWithData( 0L, bmask, rowBytes*h, 0L);
-  CGImageRef id = CGImageMaskCreate( w, h, 1, 1, rowBytes, srcp, 0L, false);
+  CGImageRef id_ = CGImageMaskCreate( w, h, 1, 1, rowBytes, srcp, 0L, false);
   CGDataProviderRelease(srcp);
-  return (Fl_Bitmask)id;
+  return (Fl_Bitmask)id_;
 }
-void fl_delete_bitmask(Fl_Bitmask id) {
-  if (id) CGImageRelease((CGImageRef)id);
+void fl_delete_bitmask(Fl_Bitmask bm) {
+  if (bm) CGImageRelease((CGImageRef)bm);
 }
+
+
 #elif defined(WIN32) // Windows bitmask functions...
+
+
 // 'fl_create_bitmap()' - Create a 1-bit bitmap for drawing...
 static Fl_Bitmask fl_create_bitmap(int w, int h, const uchar *data) {
   // we need to pad the lines out to words & swap the bits
@@ -108,7 +66,7 @@ static Fl_Bitmask fl_create_bitmap(int w, int h, const uchar *data) {
   uchar* newarray = new uchar[w2*h];
   const uchar* src = data;
   uchar* dest = newarray;
-  Fl_Bitmask id;
+  Fl_Bitmask bm;
   static uchar reverse[16] =	/* Bit reversal lookup table */
   	      { 0x00, 0x88, 0x44, 0xcc, 0x22, 0xaa, 0x66, 0xee,
 		0x11, 0x99, 0x55, 0xdd, 0x33, 0xbb, 0x77, 0xff };
@@ -120,18 +78,18 @@ static Fl_Bitmask fl_create_bitmap(int w, int h, const uchar *data) {
     dest += w2-w1;
   }
 
-  id = CreateBitmap(w, h, 1, 1, newarray);
+  bm = CreateBitmap(w, h, 1, 1, newarray);
 
   delete[] newarray;
 
-  return id;
+  return bm;
 }
 
 // 'fl_create_bitmask()' - Create an N-bit bitmap for masking...
 Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *data) {
   // this won't work when the user changes display mode during run or
   // has two screens with differnet depths
-  Fl_Bitmask id;
+  Fl_Bitmask bm;
   static uchar hiNibble[16] =
   { 0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
     0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0 };
@@ -180,45 +138,21 @@ Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *data) {
     dst += pad;
   }
 
-  id = CreateBitmap(w, h, np, bpp, newarray);
+  bm = CreateBitmap(w, h, np, bpp, newarray);
   delete[] newarray;
 
-  return id;
+  return bm;
 }
 
-#if 0 // This doesn't appear to be used anywhere...
-Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *data, int for_mask) {
-  // we need to pad the lines out to words & swap the bits
-  // in each byte.
-  int w1 = (w+7)/8;
-  int w2 = ((w+15)/16)*2;
-  uchar* newarray = new uchar[w2*h];
-  const uchar* src = data;
-  uchar* dest = newarray;
-  Fl_Bitmask id;
-  static uchar reverse[16] =	/* Bit reversal lookup table */
-  	      { 0x00, 0x88, 0x44, 0xcc, 0x22, 0xaa, 0x66, 0xee,
-		0x11, 0x99, 0x55, 0xdd, 0x33, 0xbb, 0x77, 0xff };
-
-  for (int y=0; y < h; y++) {
-    for (int n = 0; n < w1; n++, src++)
-      *dest++ = (reverse[*src & 0x0f] & 0xf0) |
-	        (reverse[(*src >> 4) & 0x0f] & 0x0f);
-    dest += w2-w1;
-  }
-
-  id = CreateBitmap(w, h, 1, 1, newarray);
-
-  delete[] newarray;
-
-  return (id);
-}
-#  endif // 0
 
 void fl_delete_bitmask(Fl_Bitmask bm) {
   DeleteObject((HGDIOBJ)bm);
 }
+
+
 #else // X11 bitmask functions
+
+
 Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *data) {
   return XCreateBitmapFromData(fl_display, fl_window, (const char *)data,
                                (w+7)&-8, h);
@@ -227,60 +161,14 @@ Fl_Bitmask fl_create_bitmask(int w, int h, const uchar *data) {
 void fl_delete_bitmask(Fl_Bitmask bm) {
   fl_delete_offscreen((Fl_Offscreen)bm);
 }
+
+
 #endif // __APPLE__
 
 
-// MRS: Currently it appears that CopyDeepMask() does not work with an 8-bit alpha mask.
-//      If you want to test/fix this, uncomment the "#ifdef __APPLE__" and comment out
-//      the "#if 0" here.  Also see Fl_Image.cxx for a similar check...
-
-//#ifdef __APPLE_QD__
-#if 0
-// Create an 8-bit mask used for alpha blending
-Fl_Bitmask fl_create_alphamask(int w, int h, int d, int ld, const uchar *array) {
-  Rect srcRect;
-  srcRect.left = 0; srcRect.right = w;
-  srcRect.top = 0; srcRect.bottom = h;
-  GrafPtr savePort;
-
-  GetPort(&savePort); // remember the current port
-
-  Fl_Bitmask gw;
-  NewGWorld( &gw, 8, &srcRect, 0L, 0L, 0 );
-  PixMapHandle pm = GetGWorldPixMap( gw );
-  if ( pm ) 
-  {
-    LockPixels( pm );
-    if ( *pm ) 
-    {
-      uchar *base = (uchar*)GetPixBaseAddr( pm );
-      if ( base ) 
-      {
-        PixMapPtr pmp = *pm;
-        // verify the parameters for direct memory write
-        if ( pmp->pixelType == 0 || pmp->pixelSize == 8 || pmp->cmpCount == 1 || pmp->cmpSize == 8 ) 
-        {
-	  // Copy alpha values from the source array to the pixmap...
-	  array += d - 1;
-          int rowoffset = (pmp->rowBytes & 0x3fff) - w;
-	  for (int y = h; y > 0; y --, array += ld, base += rowoffset) {
-	    for (int x = w; x > 0; x --, array += d) {
-	      *base++ = 255 /*255 - *array*/;
-	    }
-	  }
-        }
-      }
-      UnlockPixels( pm );
-    }
-  }
-
-  SetPort(savePort);
-  return gw;               /* tell caller we succeeded! */
-}
-#else
 // Create a 1-bit mask used for alpha blending
 Fl_Bitmask fl_create_alphamask(int w, int h, int d, int ld, const uchar *array) {
-  Fl_Bitmask mask;
+  Fl_Bitmask bm;
   int bmw = (w + 7) / 8;
   uchar *bitmap = new uchar[bmw * h];
   uchar *bitptr, bit;
@@ -345,84 +233,152 @@ Fl_Bitmask fl_create_alphamask(int w, int h, int d, int ld, const uchar *array)
       }
     }
 
-  mask = fl_create_bitmask(w, h, bitmap);
+  bm = fl_create_bitmask(w, h, bitmap);
   delete[] bitmap;
 
-  return (mask);
+  return (bm);
 }
-#endif // __APPLE__
 
 void Fl_Bitmap::draw(int XP, int YP, int WP, int HP, int cx, int cy) {
+  fl_graphics_driver->draw(this, XP, YP, WP, HP, cx, cy);
+}
+
+int Fl_Bitmap::start(int XP, int YP, int WP, int HP, int &cx, int &cy,
+		 int &X, int &Y, int &W, int &H)
+{
   if (!array) {
     draw_empty(XP, YP);
-    return;
+    return 1;
   }
-
   // account for current clip region (faster on Irix):
-  int X,Y,W,H; fl_clip_box(XP,YP,WP,HP,X,Y,W,H);
+  fl_clip_box(XP,YP,WP,HP,X,Y,W,H);
   cx += X-XP; cy += Y-YP;
   // clip the box down to the size of image, quit if empty:
   if (cx < 0) {W += cx; X -= cx; cx = 0;}
-  if ((cx+W) > w()) W = w()-cx;
-  if (W <= 0) return;
+  if (cx+W > w()) W = w()-cx;
+  if (W <= 0) return 1;
   if (cy < 0) {H += cy; Y -= cy; cy = 0;}
-  if ((cy+H) > h()) H = h()-cy;
-  if (H <= 0) return;
-#ifdef WIN32
-  if (!id) id = fl_create_bitmap(w(), h(), array);
+  if (cy+H > h()) H = h()-cy;
+  if (H <= 0) return 1;
+#if defined(WIN32)
+  if (!id_) id_ = fl_create_bitmap(w(), h(), array);
+#else
+  if (!id_) id_ = fl_create_bitmask(w(), h(), array);
+#endif
+  return 0;
+}
 
+#ifdef __APPLE__
+void Fl_Quartz_Graphics_Driver::draw(Fl_Bitmap *bm, int XP, int YP, int WP, int HP, int cx, int cy) {
+  int X, Y, W, H;
+  if (bm->start(XP, YP, WP, HP, cx, cy, X, Y, W, H)) {
+    return;
+  }
+  if (bm->id_ && fl_gc) {
+    CGRect rect = { { (CGFloat)X, (CGFloat)Y }, { (CGFloat)W, (CGFloat)H } };
+    Fl_X::q_begin_image(rect, cx, cy, bm->w(), bm->h());
+    CGContextDrawImage(fl_gc, rect, (CGImageRef)bm->id_);
+    Fl_X::q_end_image();
+  }
+}
+
+#elif defined(WIN32)
+void Fl_GDI_Graphics_Driver::draw(Fl_Bitmap *bm, int XP, int YP, int WP, int HP, int cx, int cy) {
+  int X, Y, W, H;
+  if (bm->start(XP, YP, WP, HP, cx, cy, X, Y, W, H)) {
+    return;
+  }
+  
   HDC tempdc = CreateCompatibleDC(fl_gc);
   int save = SaveDC(tempdc);
-  SelectObject(tempdc, (HGDIOBJ)id);
+  SelectObject(tempdc, (HGDIOBJ)bm->id_);
   SelectObject(fl_gc, fl_brush());
   // secret bitblt code found in old MSWindows reference manual:
   BitBlt(fl_gc, X, Y, W, H, tempdc, cx, cy, 0xE20746L);
   RestoreDC(tempdc, save);
   DeleteDC(tempdc);
-#elif defined(__APPLE_QD__)
-  if (!id) id = fl_create_bitmask(w(), h(), array);
-  GrafPtr dstPort;
-  GetPort( &dstPort );
-  Rect src, dst;
-  GetPortBounds( (Fl_Offscreen)id, &src );
-  SetRect( &src, cx, cy, cx+W, cy+H );
-  SetRect( &dst, X, Y, X+W, Y+H );
-  CopyBits(GetPortBitMapForCopyBits((Fl_Offscreen)id),	// srcBits
-	   GetPortBitMapForCopyBits(dstPort),	// dstBits
-	   &src,		 		// src bounds
-	   &dst, 				// dst bounds
-	   srcOr, 				// mode
-	   0L);					// mask region
-#elif defined(__APPLE_QUARTZ__)
-  if (!id) id = fl_create_bitmask(w(), h(), array);
-  if (id && fl_gc) {
-    CGRect rect = { { X, Y }, { W, H } };
-    Fl_X::q_begin_image(rect, cx, cy, w(), h());
-    CGContextDrawImage(fl_gc, rect, (CGImageRef)id);
-    Fl_X::q_end_image();
-  }
-#else
-  if (!id) id = fl_create_bitmask(w(), h(), array);
+}  
 
-  XSetStipple(fl_display, fl_gc, id);
-  int ox = X-cx; if (ox < 0) ox += w();
-  int oy = Y-cy; if (oy < 0) oy += h();
+void Fl_GDI_Printer_Graphics_Driver::draw(Fl_Bitmap *bm, int XP, int YP, int WP, int HP, int cx, int cy) {
+  int X, Y, W, H;
+  typedef BOOL (WINAPI* fl_transp_func)  (HDC,int,int,int,int,HDC,int,int,int,int,UINT);
+  static fl_transp_func fl_TransparentBlt = NULL;
+  static HMODULE hMod = NULL;
+  if (!hMod) {
+    hMod = LoadLibrary("MSIMG32.DLL");
+    if (hMod) fl_TransparentBlt = (fl_transp_func)GetProcAddress(hMod, "TransparentBlt");
+  }
+  if (!fl_TransparentBlt) {
+    Fl_GDI_Graphics_Driver::draw(bm,  XP,  YP,  WP,  HP,  cx,  cy);
+    return;
+    }
+  if (bm->start(XP, YP, WP, HP, cx, cy, X, Y, W, H)) {
+    return;
+  }
+  
+  HDC tempdc;
+  int save;
+  // algorithm for bitmap output to Fl_GDI_Printer
+  Fl_Color save_c = fl_color(); // save bitmap's desired color
+  uchar r, g, b;
+  Fl::get_color(save_c, r, g, b);
+  r = 255-r;
+  g = 255-g;
+  b = 255-b;
+  Fl_Color background = fl_rgb_color(r, g, b); // a color very different from the bitmap's
+  Fl_Offscreen tmp_id = fl_create_offscreen(W, H);
+  fl_begin_offscreen(tmp_id);
+  fl_color(background);
+  fl_rectf(0,0,W,H); // use this color as offscreen background
+  fl_color(save_c); // back to bitmap's color
+  tempdc = CreateCompatibleDC(fl_gc);
+  save = SaveDC(tempdc);
+  SelectObject(tempdc, (HGDIOBJ)bm->id_);
+  SelectObject(fl_gc, fl_brush()); // use bitmap's desired color
+  BitBlt(fl_gc, 0, 0, W, H, tempdc, 0, 0, 0xE20746L); // draw bitmap to offscreen
+  fl_end_offscreen(); // offscreen data is in tmp_id
+  SelectObject(tempdc, (HGDIOBJ)tmp_id); // use offscreen data
+  // draw it to printer context with background color as transparent
+  fl_TransparentBlt(fl_gc, X,Y,W,H, tempdc, cx, cy, bm->w(), bm->h(), RGB(r, g, b) ); 
+  fl_delete_offscreen(tmp_id);
+  RestoreDC(tempdc, save);
+  DeleteDC(tempdc);
+}  
+
+#else // Xlib
+void Fl_Xlib_Graphics_Driver::draw(Fl_Bitmap *bm, int XP, int YP, int WP, int HP, int cx, int cy) {
+  int X, Y, W, H;
+  if (bm->start(XP, YP, WP, HP, cx, cy, X, Y, W, H)) {
+    return;
+  }
+  
+  XSetStipple(fl_display, fl_gc, bm->id_);
+  int ox = X-cx; if (ox < 0) ox += bm->w();
+  int oy = Y-cy; if (oy < 0) oy += bm->h();
   XSetTSOrigin(fl_display, fl_gc, ox, oy);
   XSetFillStyle(fl_display, fl_gc, FillStippled);
   XFillRectangle(fl_display, fl_window, fl_gc, X, Y, W, H);
   XSetFillStyle(fl_display, fl_gc, FillSolid);
-#endif
 }
+#endif
 
+/**
+  The destructor frees all memory and server resources that are used by
+  the bitmap.
+*/
 Fl_Bitmap::~Fl_Bitmap() {
   uncache();
   if (alloc_array) delete[] (uchar *)array;
 }
 
 void Fl_Bitmap::uncache() {
-  if (id) {
-    fl_delete_bitmask((Fl_Offscreen)id);
-    id = 0;
+  if (id_) {
+#ifdef __APPLE_QUARTZ__
+    fl_delete_bitmask((Fl_Bitmask)id_);
+#else
+    fl_delete_bitmask((Fl_Offscreen)id_);
+#endif
+    id_ = 0;
   }
 }
 
@@ -463,7 +419,7 @@ Fl_Image *Fl_Bitmap::copy(int W, int H) {
 		xstep, ystep;	// X & Y step increments
 
 
-  // Figure out Bresenheim step/modulus values...
+  // Figure out Bresenham step/modulus values...
   xmod   = w() % W;
   xstep  = w() / W;
   ymod   = h() % H;
@@ -514,5 +470,5 @@ Fl_Image *Fl_Bitmap::copy(int W, int H) {
 
 
 //
-// End of "$Id: Fl_Bitmap.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id$".
 //
