@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Menu_Window.cxx 8198 2011-01-06 10:24:58Z manolo $"
+// "$Id$"
 //
 // Menu window code for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -63,6 +54,7 @@ void Fl_Menu_Window::show() {
 }
 
 void Fl_Menu_Window::flush() {
+  if (!shown()) return;
 #if HAVE_OVERLAY
   if (!fl_overlay_visual || !overlay()) {Fl_Single_Window::flush(); return;}
   Fl_X *myi = Fl_X::i(this);
@@ -105,6 +97,20 @@ Fl_Menu_Window::~Fl_Menu_Window() {
   hide();
 }
 
+
+Fl_Menu_Window::Fl_Menu_Window(int W, int H, const char *l)
+: Fl_Single_Window(W,H,l) 
+{ 
+  image(0); 
+}
+
+
+Fl_Menu_Window::Fl_Menu_Window(int X, int Y, int W, int H, const char *l)
+: Fl_Single_Window(X,Y,W,H,l) { 
+  image(0); 
+}
+
+
 //
-// End of "$Id: Fl_Menu_Window.cxx 8198 2011-01-06 10:24:58Z manolo $".
+// End of "$Id$".
 //
