@@ -1,24 +1,15 @@
 //
-// "$Id: fl_engraved_label.cxx 5190 2006-06-09 16:16:34Z mike $"
+// "$Id$"
 //
 // Engraved label drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2011 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -35,11 +26,11 @@
 
 static void innards(
     const Fl_Label* o, int X, int Y, int W, int H, Fl_Align align,
-    int data[][3], int n)
+    const int data[][3], int n)
 {
   Fl_Align a1 = align;
   if (a1 & FL_ALIGN_CLIP) {
-    fl_clip(X, Y, W, H); a1 = (Fl_Align)(a1&~FL_ALIGN_CLIP);}
+    fl_push_clip(X, Y, W, H); a1 = (Fl_Align)(a1&~FL_ALIGN_CLIP);}
   fl_font((Fl_Font)o->font, o->size);
   for (int i = 0; i < n; i++) {
     fl_color((Fl_Color)(i < n-1 ? data[i][2] : o->color));
@@ -51,14 +42,14 @@ static void innards(
 static void fl_shadow_label(
     const Fl_Label* o, int X, int Y, int W, int H, Fl_Align align)
 {
-  static int data[2][3] = {{2,2,FL_DARK3},{0,0,0}};
+  static const int data[2][3] = {{2,2,FL_DARK3},{0,0,0}};
   innards(o, X, Y, W, H, align, data, 2);
 }
 
 static void fl_engraved_label(
     const Fl_Label* o, int X, int Y, int W, int H, Fl_Align align)
 {
-  static int data[7][3] = {
+  static const int data[7][3] = {
     {1,0,FL_LIGHT3},{1,1,FL_LIGHT3},{0,1,FL_LIGHT3},
     {-1,0,FL_DARK3},{-1,-1,FL_DARK3},{0,-1,FL_DARK3},
     {0,0,0}};
@@ -68,7 +59,7 @@ static void fl_engraved_label(
 static void fl_embossed_label(
     const Fl_Label* o, int X, int Y, int W, int H, Fl_Align align)
 {
-  static int data[7][3] = {
+  static const int data[7][3] = {
     {-1,0,FL_LIGHT3},{-1,-1,FL_LIGHT3},{0,-1,FL_LIGHT3},
     {1,0,FL_DARK3},{1,1,FL_DARK3},{0,1,FL_DARK3},
     {0,0,0}};
@@ -89,5 +80,5 @@ Fl_Labeltype fl_define_FL_EMBOSSED_LABEL() {
 }
 
 //
-// End of "$Id: fl_engraved_label.cxx 5190 2006-06-09 16:16:34Z mike $".
+// End of "$Id$".
 //
