@@ -1,9 +1,9 @@
 //
-// "$Id: gl_draw.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
+// "$Id$"
 //
 // OpenGL drawing support routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2011 by Bill Spitzak and others.
+// Copyright 1998-2018 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -24,6 +24,7 @@
 
 #include <FL/Fl.H>
 #include <FL/gl.h>
+#include <FL/gl_draw.H>
 #include <FL/x.H>
 #include <FL/fl_draw.H>
 #include <FL/Fl_Device.H>
@@ -32,12 +33,22 @@
 #include <FL/fl_utf8.h>
 
 #if !defined(WIN32) && !defined(__APPLE__)
-#include <FL/Xutf8.h>
+#include "Xutf8.h"
 #endif
 
-#if USE_XFT
-//extern XFontStruct* fl_xxfont();
-#endif // USE_XFT
+#if defined(__APPLE__)
+
+#if !defined(kCGBitmapByteOrder32Host) // doc says available 10.4 but some 10.4 don't have it
+#  define kCGBitmapByteOrder32Host 0
+#endif
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_4
+#  include <OpenGL/glext.h>
+#  define GL_TEXTURE_RECTANGLE_ARB GL_TEXTURE_RECTANGLE_EXT
+#  include <FL/Fl_Gl_Window.H>
+#endif // < MAC_OS_X_VERSION_10_4
+
+#endif // __APPLE__
 
 /** Returns the current font's height */
 int   gl_height() {return fl_height();}
@@ -52,11 +63,9 @@ double gl_width(uchar c) {return fl_width(c);}
 
 static Fl_Font_Descriptor *gl_fontsize;
 
-#define GL_DRAW_USES_TEXTURES  (defined(__APPLE__) && !__ppc__) // 1 only for non-PPC OSX
+#define GENLISTSIZE 256
 #ifndef __APPLE__
 #  define USE_OksiD_style_GL_font_selection 1  // Most hosts except OSX
-#else
-#  undef USE_OksiD_style_GL_font_selection  // OSX
 #endif
 
 #if USE_XFT
@@ -69,11 +78,13 @@ static Fl_Font_Descriptor *gl_fontsize;
 void  gl_font(int fontid, int size) {
   fl_font(fontid, size);
   Fl_Font_Descriptor *fl_fontsize = fl_graphics_driver->font_descriptor();
-#if !GL_DRAW_USES_TEXTURES
+#ifndef __APPLE__
   if (!fl_fontsize->listbase) {
 
 #ifdef  USE_OksiD_style_GL_font_selection
-    fl_fontsize->listbase = glGenLists(0x10000);
+#undef GENLISTSIZE
+#define GENLISTSIZE 0x10000
+    fl_fontsize->listbase = glGenLists(GENLISTSIZE);
 #else // Fltk-1.1.8 style GL font selection
 
 #if defined (USE_X11) // X-windows options follow, either XFT or "plain" X
@@ -83,44 +94,30 @@ void  gl_font(int fontid, int size) {
  * then sorting through them at draw time (for normal X rendering) to find which one can
  * render the current glyph... But for now, just use the first font in the list for GL...
  */
-    XFontStruct *font = fl_xfont;
+    XFontStruct *font = fl_X_core_font();
     int base = font->min_char_or_byte2;
     int count = font->max_char_or_byte2-base+1;
-    fl_fontsize->listbase = glGenLists(256);
+    fl_fontsize->listbase = glGenLists(GENLISTSIZE);
     glXUseXFont(font->fid, base, count, fl_fontsize->listbase+base);
 # elif defined(WIN32)
     // this is unused because USE_OksiD_style_GL_font_selection == 1
     int base = fl_fontsize->metr.tmFirstChar;
     int count = fl_fontsize->metr.tmLastChar-base+1;
     HFONT oldFid = (HFONT)SelectObject(fl_gc, fl_fontsize->fid);
-    fl_fontsize->listbase = glGenLists(256);
+    fl_fontsize->listbase = glGenLists(GENLISTSIZE);
     wglUseFontBitmaps(fl_gc, base, count, fl_fontsize->listbase+base);
     SelectObject(fl_gc, oldFid);
-# elif defined(__APPLE_QUARTZ__)
-//AGL is not supported for use in 64-bit applications:
-//http://developer.apple.com/mac/library/documentation/Carbon/Conceptual/Carbon64BitGuide/OtherAPIChanges/OtherAPIChanges.html
-    short font, face, size;
-    uchar fn[256];
-    fn[0]=strlen(fl_fontsize->q_name);
-    strcpy((char*)(fn+1), fl_fontsize->q_name);
-    GetFNum(fn, &font);
-    face = 0;
-    size = fl_fontsize->size;
-    fl_fontsize->listbase = glGenLists(256);
-	aglUseFont(aglGetCurrentContext(), font, face,
-               size, 0, 256, fl_fontsize->listbase);
-# else 
-#   error unsupported platform
 # endif
 
 #endif // USE_OksiD_style_GL_font_selection
   }
   glListBase(fl_fontsize->listbase);
-#endif // !GL_DRAW_USES_TEXTURES
+#endif // !__APPLE__
   gl_fontsize = fl_fontsize;
 }
 
-#ifndef __APPLE__
+#if !defined(__APPLE__)
+#if !(defined(USE_X11) || USE_XFT)
 static void get_list(int r) {
   gl_fontsize->glok[r] = 1;
 #if defined(USE_X11)
@@ -131,7 +128,7 @@ static void get_list(int r) {
   for (int i = 0; i < 0x400; i++) {
     XFontStruct *font = NULL;
     unsigned short id;
-    XGetUtf8FontAndGlyph(gl_fontsize->font, ii, &font, &id);
+    fl_XGetUtf8FontAndGlyph(gl_fontsize->font, ii, &font, &id);
     if (font) glXUseXFont(font->fid, id, 1, gl_fontsize->listbase+ii);
     ii++;
    }
@@ -139,15 +136,14 @@ static void get_list(int r) {
 #elif defined(WIN32)
   unsigned int ii = r * 0x400;
   HFONT oldFid = (HFONT)SelectObject(fl_gc, gl_fontsize->fid);
-  wglUseFontBitmapsW(fl_gc, ii, ii + 0x03ff, gl_fontsize->listbase+ii);
+  wglUseFontBitmapsW(fl_gc, ii, 0x400, gl_fontsize->listbase+ii);
   SelectObject(fl_gc, oldFid);
-#elif defined(__APPLE_QUARTZ__)
-// handled by textures
 #else
 #  error unsupported platform
 #endif
 } // get_list
-#endif
+#endif // !(defined(USE_X11) || USE_XFT)
+#endif // !defined(__APPLE__)
 
 void gl_remove_displaylist_fonts()
 {
@@ -170,8 +166,8 @@ void gl_remove_displaylist_fonts()
           past->next = f->next;
         }
 
-        // It would be nice if this next line was in a desctructor somewhere
-        glDeleteLists(f->listbase, 256);
+        // It would be nice if this next line was in a destructor somewhere
+        glDeleteLists(f->listbase, GENLISTSIZE);
 
         Fl_Font_Descriptor* tmp = f;
         f = f->next;
@@ -187,7 +183,7 @@ void gl_remove_displaylist_fonts()
 #endif
 }
 
-#if GL_DRAW_USES_TEXTURES
+#ifdef __APPLE__
 static void gl_draw_textures(const char* str, int n);
 #endif
 
@@ -197,37 +193,31 @@ static void gl_draw_textures(const char* str, int n);
  \see On the Mac OS X platform, see gl_texture_pile_height(int)
   */
 void gl_draw(const char* str, int n) {
-#ifdef __APPLE__  
-  
-#if GL_DRAW_USES_TEXTURES
+#ifdef __APPLE__
   gl_draw_textures(str, n);
 #else
-  glCallLists(n, GL_UNSIGNED_BYTE, str);
-#endif
-  
-#else
-  static xchar *buf = NULL;
-  static int l = 0;
-  int wn = fl_utf8toUtf16(str, n, (unsigned short*)buf, l);
-  if(wn >= l) {
-    buf = (xchar*) realloc(buf, sizeof(xchar) * (wn + 1));
+  static unsigned short *buf = NULL;
+  static unsigned l = 0;
+  unsigned wn = fl_utf8toUtf16(str, n, buf, l);
+  if (wn >= l) {
+    buf = (unsigned short*) realloc(buf, sizeof(unsigned short) * (wn + 1));
     l = wn + 1;
-    wn = fl_utf8toUtf16(str, n, (unsigned short*)buf, l);
+    wn = fl_utf8toUtf16(str, n, buf, l);
   }
-  n = wn;
 
-  int i;
-  for (i = 0; i < n; i++) {
+#if !(defined(USE_X11) || USE_XFT)
+  for (unsigned i = 0; i < wn; i++) {
     unsigned int r;
-    r = (str[i] & 0xFC00) >> 10;
+    r = (buf[i] & 0xFC00) >> 10;
     if (!gl_fontsize->glok[r]) get_list(r);
   }
-  glCallLists(n, GL_UNSIGNED_SHORT, buf);
+#endif
+  glCallLists(wn, GL_UNSIGNED_SHORT, buf);
 #endif
 }
 
 /**
-  Draws n characters of the string in the current font at the given position 
+  Draws n characters of the string in the current font at the given position
  \see On the Mac OS X platform, see gl_texture_pile_height(int)
   */
 void gl_draw(const char* str, int n, int x, int y) {
@@ -236,7 +226,7 @@ void gl_draw(const char* str, int n, int x, int y) {
 }
 
 /**
-  Draws n characters of the string in the current font at the given position 
+  Draws n characters of the string in the current font at the given position
  \see On the Mac OS X platform, see gl_texture_pile_height(int)
   */
 void gl_draw(const char* str, int n, float x, float y) {
@@ -253,7 +243,7 @@ void gl_draw(const char* str) {
 }
 
 /**
-  Draws a nul-terminated string in the current font at the given position 
+  Draws a nul-terminated string in the current font at the given position
  \see On the Mac OS X platform, see gl_texture_pile_height(int)
   */
 void gl_draw(const char* str, int x, int y) {
@@ -261,7 +251,7 @@ void gl_draw(const char* str, int x, int y) {
 }
 
 /**
-  Draws a nul-terminated string in the current font at the given position 
+  Draws a nul-terminated string in the current font at the given position
  \see On the Mac OS X platform, see gl_texture_pile_height(int)
   */
 void gl_draw(const char* str, float x, float y) {
@@ -282,11 +272,13 @@ void gl_draw(
   const char* str, 	// the (multi-line) string
   int x, int y, int w, int h, 	// bounding box
   Fl_Align align) {
-  fl_draw(str, x, -y-h, w, h, align, gl_draw_invert);
+  fl_draw(str, x, -y-h, w, h, align, gl_draw_invert, NULL, 0);
 }
 
 /** Measure how wide and tall the string will be when drawn by the gl_draw() function */
-void gl_measure(const char* str, int& x, int& y) {fl_measure(str,x,y);}
+void gl_measure(const char* str, int& x, int& y) {
+  fl_measure(str,x,y,0);
+}
 
 /**
   Outlines the given rectangle with the current color.
@@ -348,9 +340,19 @@ void gl_draw_image(const uchar* b, int x, int y, int w, int h, int d, int ld) {
   glDrawPixels(w,h,d<4?GL_RGB:GL_RGBA,GL_UNSIGNED_BYTE,(const ulong*)b);
 }
 
-#if GL_DRAW_USES_TEXTURES || defined(FL_DOXYGEN)
+#if defined(__APPLE__) || defined(FL_DOXYGEN)
+/* Text drawing to an OpenGL scene under Mac OS X is implemented using textures, as recommended by Apple.
+ This allows to use any font at any size, and any Unicode character.
+ Some old Apple hardware doesn't implement the required GL_EXT_texture_rectangle extension.
+ For these, glutStrokeString() is used to draw text. In that case, it's possible to vary text size,
+ but not text font, and only ASCII characters can be drawn.
+ */
 
-#include <FL/glu.h>
+static int gl_scale = 1; // set to 2 for high resolution Fl_Gl_Window
+static int has_texture_rectangle = 0; // true means GL_EXT_texture_rectangle is available
+
+#include <FL/glu.h>  // for gluUnProject() and gluCheckExtension()
+#include <FL/glut.H> // for glutStrokeString() and glutStrokeLength()
 
 // manages a fifo pile of pre-computed string textures
 class gl_texture_fifo {
@@ -361,13 +363,15 @@ private:
     char *utf8; //its text
     Fl_Font_Descriptor *fdesc; // its font
     int width; // its width
-    int height; // its height
+    float ratio; // used without rectangle texture
+    int scale; // 1 or 2 for low/high resolution
+    Fl_Color color;
   } data;
   data *fifo; // array of pile elements
   int size_; // pile height
   int current; // the oldest texture to have entered the pile
   int last; // pile top
-  int textures_generated; // true iff glGenTextures has been called
+  int textures_generated; // true after glGenTextures has been called
   void display_texture(int rank);
   int compute_texture(const char* str, int n);
   int already_known(const char *str, int n);
@@ -406,44 +410,60 @@ void gl_texture_fifo::display_texture(int rank)
   glMatrixMode (GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity ();
-  float winw = Fl_Window::current()->w();
-  float winh = Fl_Window::current()->h();
-  glScalef (2.0f / winw, 2.0f /  winh, 1.0f);
-  glTranslatef (-winw / 2.0f, -winh / 2.0f, 0.0f);
-  //write the texture on screen
-  GLfloat pos[4];
-  glGetFloatv(GL_CURRENT_RASTER_POSITION, pos);
-  CGRect bounds = CGRectMake (pos[0], pos[1] - fl_descent(), fifo[rank].width, fifo[rank].height);
-  
+  float winw = gl_scale * Fl_Window::current()->w();
+  float winh = gl_scale * Fl_Window::current()->h();
   // GL_COLOR_BUFFER_BIT for glBlendFunc, GL_ENABLE_BIT for glEnable / glDisable
-  glPushAttrib(GL_ENABLE_BIT | GL_TEXTURE_BIT | GL_COLOR_BUFFER_BIT); 
+  glPushAttrib(GL_ENABLE_BIT | GL_TEXTURE_BIT | GL_COLOR_BUFFER_BIT);
   glDisable (GL_DEPTH_TEST); // ensure text is not removed by depth buffer test.
   glEnable (GL_BLEND); // for text fading
-  glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA); // ditto
-  glEnable (GL_TEXTURE_RECTANGLE_EXT);	
+  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // ditto
   glDisable(GL_LIGHTING);
-  glBindTexture (GL_TEXTURE_RECTANGLE_EXT, fifo[rank].texName);
-  glBegin (GL_QUADS);
-  glTexCoord2f (0.0f, 0.0f); // draw lower left in world coordinates
-  glVertex2f (bounds.origin.x, bounds.origin.y);
-  
-  glTexCoord2f (0.0f, fifo[rank].height); // draw upper left in world coordinates
-  glVertex2f (bounds.origin.x, bounds.origin.y + bounds.size.height);
-  
-  glTexCoord2f (fifo[rank].width, fifo[rank].height); // draw upper right in world coordinates
-  glVertex2f (bounds.origin.x + bounds.size.width, bounds.origin.y + bounds.size.height);
-  
-  glTexCoord2f (fifo[rank].width, 0.0f); // draw lower right in world coordinates
-  glVertex2f (bounds.origin.x + bounds.size.width, bounds.origin.y);
-  glEnd ();
+  GLfloat pos[4];
+  glGetFloatv(GL_CURRENT_RASTER_POSITION, pos);
+  float R = 2;
+  if (!has_texture_rectangle) {
+    R *= fifo[rank].ratio;
+  }
+  glScalef (R/winw, R/winh, 1.0f);
+  glTranslatef (-winw/R, -winh/R, 0.0f);
+  if (has_texture_rectangle) {
+    // The texture map is already colored, but we still need to apply alpha.
+    GLfloat colors[4];
+    glGetFloatv(GL_CURRENT_COLOR, colors);
+    glColor4f(1.0f, 1.0f, 1.0f, colors[3]);
+    glEnable (GL_TEXTURE_RECTANGLE_ARB);
+    glBindTexture (GL_TEXTURE_RECTANGLE_ARB, fifo[rank].texName);
+    GLint height;
+    glGetTexLevelParameteriv(GL_TEXTURE_RECTANGLE_ARB, 0, GL_TEXTURE_HEIGHT, &height);
+    CGRect bounds = CGRectMake (pos[0], pos[1] - gl_scale*fl_descent(), fifo[rank].width, height);
+    //write the texture on screen
+    glBegin (GL_QUADS);
+    glTexCoord2f (0.0f, 0.0f); // draw lower left in world coordinates
+    glVertex2f (bounds.origin.x, bounds.origin.y);
+
+    glTexCoord2f (0.0f, height); // draw upper left in world coordinates
+    glVertex2f (bounds.origin.x, bounds.origin.y + bounds.size.height);
+
+    glTexCoord2f (fifo[rank].width, height); // draw upper right in world coordinates
+    glVertex2f (bounds.origin.x + bounds.size.width, bounds.origin.y + bounds.size.height);
+
+    glTexCoord2f (fifo[rank].width, 0.0f); // draw lower right in world coordinates
+    glVertex2f (bounds.origin.x + bounds.size.width, bounds.origin.y);
+    // Restore the current color value.
+    glColor4fv(colors);
+    glEnd ();
+  } else {
+    glTranslatef(pos[0]*2/R, pos[1]*2/R, 0.0);
+    glutStrokeString(GLUT_STROKE_ROMAN, (uchar*)fifo[rank].utf8);
+  }
   glPopAttrib();
-  
+
   // reset original matrices
   glPopMatrix(); // GL_MODELVIEW
   glMatrixMode (GL_PROJECTION);
   glPopMatrix();
   glMatrixMode (matrixMode);
-  
+
   //set the raster position to end of string
   pos[0] += fifo[rank].width;
   GLdouble modelmat[16];
@@ -462,34 +482,53 @@ int gl_texture_fifo::compute_texture(const char* str, int n)
 {
   current = (current + 1) % size_;
   if (current > last) last = current;
-  //write str to a bitmap just big enough  
   if ( fifo[current].utf8 ) free(fifo[current].utf8);
   fifo[current].utf8 = (char *)malloc(n + 1);
   memcpy(fifo[current].utf8, str, n);
   fifo[current].utf8[n] = 0;
-  fifo[current].width = 0, fifo[current].height = 0;
-  fl_measure(fifo[current].utf8, fifo[current].width, fifo[current].height, 0);
-  CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
-  void *base = calloc(4*fifo[current].width, fifo[current].height);
-  if (base == NULL) return -1;
-  fl_gc = CGBitmapContextCreate(base, fifo[current].width, fifo[current].height, 8, fifo[current].width*4, lut, kCGImageAlphaPremultipliedLast);
-  CGColorSpaceRelease(lut);
   fl_graphics_driver->font_descriptor(gl_fontsize);
-  GLfloat colors[4];
-  glGetFloatv(GL_CURRENT_COLOR, colors);
-  fl_color((uchar)(colors[0]*255), (uchar)(colors[1]*255), (uchar)(colors[2]*255));
-  fl_draw(str, n, 0, fifo[current].height - fl_descent());
-  //put this bitmap in a texture  
-  glPushAttrib(GL_TEXTURE_BIT);
-  glBindTexture (GL_TEXTURE_RECTANGLE_EXT, fifo[current].texName);
-  glTexParameteri(GL_TEXTURE_RECTANGLE_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_RECTANGLE_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexImage2D(GL_TEXTURE_RECTANGLE_EXT, 0, GL_RGBA, fifo[current].width, fifo[current].height, 0,  GL_RGBA, GL_UNSIGNED_BYTE, base);
-  glPopAttrib();
-  CGContextRelease(fl_gc);
-  fl_gc = NULL;
-  free(base);
+  int h;
+  fl_measure(fifo[current].utf8, fifo[current].width, h, 0);
+  fifo[current].width *= gl_scale;
+  h *= gl_scale;
+  fifo[current].scale = gl_scale;
   fifo[current].fdesc = gl_fontsize;
+  if (has_texture_rectangle) {
+    //write str to a bitmap just big enough
+    CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
+    void *base = NULL;
+    if (fl_mac_os_version < 100600) base = calloc(4*fifo[current].width, h);
+    CGContextRef save_gc = fl_gc;
+    // Create our bitmap in RGBA order (no matter what the host byte order is)
+    fl_gc = CGBitmapContextCreate(base, fifo[current].width, h, 8, fifo[current].width*4, lut, kCGImageAlphaPremultipliedLast);
+    CGColorSpaceRelease(lut);
+    // Note that glyphs in macOS can be colorful (Emojis, for example). So
+    // instead of caching a mask, we store the text in RGB plus A for
+    // antialiasing. Draw the text in the requested color, but don't apply user
+    // alpha yet. It is applied later at render time so it's mixed correctly
+    // with the background.
+    GLfloat colors[4];
+    glGetFloatv(GL_CURRENT_COLOR, colors);
+    Fl_Color color = ((uchar)(colors[0]*255)<<24) | ((uchar)(colors[1]*255)<<16) | ((uchar)(colors[2]*255)<<8);
+    fifo[current].color = color;
+    fl_color(color);
+    CGContextTranslateCTM(fl_gc, 0, h - gl_scale*fl_descent());
+    CGContextScaleCTM(fl_gc, gl_scale, gl_scale);
+    fl_draw(str, n, 0, 0);
+    //put this bitmap in a texture
+    glPushAttrib(GL_TEXTURE_BIT);
+    glBindTexture (GL_TEXTURE_RECTANGLE_ARB, fifo[current].texName);
+    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+    glPixelStorei(GL_UNPACK_ROW_LENGTH, fifo[current].width);
+    // Convert the created text bitmap into an RGBA OpenGL texture map.
+    glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA8, fifo[current].width, h, 0,  GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, CGBitmapContextGetData(fl_gc));
+    glPopAttrib();
+    CGContextRelease(fl_gc);
+    fl_gc = save_gc;
+    if (base) free(base);
+  } else {
+    fifo[current].ratio = float(fifo[current].width)/glutStrokeLength(GLUT_STROKE_ROMAN, (uchar*)fifo[current].utf8);
+  }
   return current;
 }
 
@@ -497,9 +536,20 @@ int gl_texture_fifo::compute_texture(const char* str, int n)
 int gl_texture_fifo::already_known(const char *str, int n)
 {
   int rank;
+  GLfloat colors[4];
+  glGetFloatv(GL_CURRENT_COLOR, colors);
+  Fl_Color color = ((uchar)(colors[0]*255)<<24) | ((uchar)(colors[1]*255)<<16) | ((uchar)(colors[2]*255)<<8);
   for ( rank = 0; rank <= last; rank++) {
-    if ( memcmp(str, fifo[rank].utf8, n) == 0 && fifo[rank].utf8[n] == 0 &&
-      fifo[rank].fdesc == gl_fontsize) return rank;
+    const data &ff = fifo[rank];
+    if (   (ff.fdesc == gl_fontsize)
+        && (ff.scale == gl_scale)
+        && (ff.color == color)
+        && (ff.utf8 != NULL)
+        && (memcmp(str, ff.utf8, n) == 0)
+        && (ff.utf8[n] == 0))
+    {
+      return rank;
+    }
   }
   return -1;
 }
@@ -507,11 +557,15 @@ int gl_texture_fifo::already_known(const char *str, int n)
 static gl_texture_fifo *gl_fifo = NULL; // points to the texture pile class instance
 
 // draws a utf8 string using pre-computed texture if available
-static void gl_draw_textures(const char* str, int n) 
+static void gl_draw_textures(const char* str, int n)
 {
+  Fl_Gl_Window *gwin = Fl_Window::current()->as_gl_window();
+  gl_scale = (gwin ? gwin->pixels_per_unit() : 1);
+  //fprintf(stderr,"gl_scale=%d\n",gl_scale);
   if (! gl_fifo) gl_fifo = new gl_texture_fifo();
   if (!gl_fifo->textures_generated) {
-    for (int i = 0; i < gl_fifo->size_; i++) glGenTextures (1, &(gl_fifo->fifo[i].texName));
+    has_texture_rectangle = gluCheckExtension((GLubyte*)"GL_EXT_texture_rectangle", glGetString(GL_EXTENSIONS));
+    if (has_texture_rectangle) for (int i = 0; i < gl_fifo->size_; i++) glGenTextures(1, &(gl_fifo->fifo[i].texName));
     gl_fifo->textures_generated = 1;
   }
   int rank = gl_fifo->already_known(str, n);
@@ -550,22 +604,14 @@ void gl_texture_pile_height(int max)
 
 /** @} */
 
-#elif defined(__APPLE__)
-// used only if __ppc__
-int gl_texture_pile_height(void) {return 0;}
-void gl_texture_pile_height(int max) {}
-#endif // GL_DRAW_USES_TEXTURES
-#if defined(__APPLE__)
 void gl_texture_reset()
 {
-#if GL_DRAW_USES_TEXTURES
   if (gl_fifo) gl_texture_pile_height(gl_texture_pile_height());
-#endif // GL_DRAW_USES_TEXTURES
 }
 #endif // __APPLE__
 
 #endif // HAVE_GL
 
 //
-// End of "$Id: gl_draw.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
+// End of "$Id$".
 //
